import fs from 'fs-extra';
import path from 'path';
import { Sade } from 'sade';
import { CLIEngine } from 'eslint';

import {
  findWorkspacePackageDir,
  findWorkspaceRootDir
} from '../../shared/utils';
import { createLintConfig } from '../../configs/lint.config';
import { lintMessage } from '../../shared/messages';
import {
  PACKAGE_JSON,
  TSCONFIG_JSON
} from '../../shared/constants/package.const';

export const lintBinCommand = (prog: Sade) => {
  prog
    .command('lint')
    .describe('Lint a package. (default lint pattern src/**/*.{js,jsx,ts,tsx}')
    // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
    // @ts-ignore
    .alias('l')
    .option('fix', 'Resolve fixable eslint errors')
    .example('lint --fix')
    .option('ignore-pattern', 'Ignore a pattern')
    .example('lint --ignore-pattern src/foo.ts')
    .action(async (opts: CLI.Options.Lint) => {
      const time = process.hrtime();
      const files = opts._.length > 0 ? opts._ : ['src/**/*.{js,jsx,ts,tsx}'];

      const project = [];
      let dir = '';
      let isRoot = false;

      try {
        dir = await findWorkspacePackageDir(true);
        project.push(path.resolve(dir, TSCONFIG_JSON));
      } catch {}

      try {
        const hasPackageDir = dir !== '';
        const rootDir = await findWorkspaceRootDir(hasPackageDir);
        isRoot = true;

        if (!hasPackageDir) {
          dir = rootDir;
        }

        project.push(path.resolve(rootDir, TSCONFIG_JSON));
      } catch {}

      const packageJsonPath = path.resolve(dir, PACKAGE_JSON);
      const { eslintConfig } = await fs.readJSON(packageJsonPath);
      const lintConfig = createLintConfig({ dir, isRoot });
      const cli = new CLIEngine({
        baseConfig: {
          ...lintConfig,
          ...(eslintConfig || {})
        },
        extensions: ['.ts', '.tsx', '.js', '.jsx'],
        fix: opts.fix,
        ignorePattern: opts['ignore-pattern'],
        parser: '@typescript-eslint/parser',
        parserOptions: {
          tsconfigRootDir: dir,
          project
        }
      });

      console.log(lintMessage.linting(files));
      const report = cli.executeOnFiles(files);

      if (opts.fix) {
        CLIEngine.outputFixes(report);
      }

      console.log(cli.getFormatter()(report.results));

      const duration = process.hrtime(time);
      console.log(lintMessage.linted(duration));

      if (report.errorCount) {
        process.exit(1);
      }
    });
};
